#!/usr/bin/perl
# $Id: rig_main.cgi,v 1.55 2015/10/06 06:06:56 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 20011             ###
###                                                      ###
############################################################
############################################################

use strict;
use warnings;
use CGI qw(:standard :netscape);
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;

my $acmode = get_cv('global','access_mode');
$acmode = 'celltest' unless ($acmode);

my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);
my $rig_setup = $rig->get_cv('main','main_page');
if ($rig_setup && -e '/home/http/cgi-bin/celltest/'.$rig_setup)
{
    &print_header('Redirecting',-onload=>'document.redirect.submit()');
    print start_form(-name=>'redirect',-action=>$rig_setup);
    &print_hidden_rig();
    print end_form();
    print end_html;
    exit;
}

my $readmode = undef;
my $usermode = undef;
my $modifymode = undef;

if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end;
}
else
{
    if (login_ok())
    { 
	$readmode = user_auth($acmode,'r');
	$usermode = user_auth($acmode,'c');
	$modifymode = user_auth($acmode,'m');
	if (RFC::RFCCGI::rig_mode())
	{
	    ####  Read and use mode deprecated after initialisation of rig authentification #### 
	    $readmode = 1;
	    $usermode = 1;
	    $modifymode = RFC::RFCCGI::rig_cert() unless ($modifymode);
	}
	####
	unless (RFC::RFCCGI::rig_auth())
	{
	    $readmode = 0;
	    $usermode = 0;
	    $modifymode = 0;
	}
        &print_header("Main page for rig ".$rig->name,-onload=>'load_start()');
        print <<END;

<script language="javascript">

function load_start()
{
    load_info();
    load_log();
    load_data();
    load_error();
    load_isiv();
}

</script>

END
        print RFC_start_html(-title=>"Main menu for rig ".$rig->name, -style=>{src=>'/celltest/RFCcontrol.css'});
        top_nav_bar_start;
	top_nav_bar_button('logbook.cgi','','','Rig '.$rig->name.' logbook','s3','rig,'.$rig->name.',1');
	if ($usermode)
	{
	    if ($rig->get_cv('userinterface','current'))
	    {
		top_nav_bar_button('start_new_test.cgi','','','Start a new test on rig'.$rig->name,'s0','rig,'.$rig->name.',1');
	    }
	    else
	    {
		top_nav_bar_button('start_new_electrode_test.cgi','','','Start a new test on rig'.$rig->name,'s0','rig,'.$rig->name.',1');
	    }
	}
	else
	{
	    top_no_button;
	}
	if ($usermode)
	{
	    top_nav_bar_button('setup_program.cgi','','','Program sequences','s3','rig,'.$rig->name.',1');
	}
	else
	{
	    top_no_button;
	}
	tab_newrow;
	if (get_CGI_value('action'))
	{
	    top_no_button;
	}
	else
	{
	    &js_back();
	    top_js_return('#006600','white');
	}
	if ($usermode)
	{
	    top_nav_bar_button('misc_conf.cgi','','','Miscellaneous setup','s1','rig,'.$rig->name.',1');
	}
	else
	{
	    my $server = get_cv('passwds','passwd_server');
	    my $taskid = &get_cv('safety_task_access','rig'.$rig->name); 
	    my @args = ('col,id,1');
	    foreach (split(/\s*\,\s*/,$taskid))
	    {
		push @args,'search_val,'.$_.',1';
	    }
	    if (CGI::https())
	    {
		top_nav_bar_button('https://'.$server.'/cgi-bin/search_safety_task.cgi','Search','dummy','Show nescesarry safety clerances for rig','s1',@args);
	    }
	    else
	    {
		top_nav_bar_button('http://'.$server.'/cgi-bin/search_safety_task.cgi','Search','dummy','Show nescesarry safety clerances for rig','s1',@args);
	    }
	}
        if (is_admin())
        {
	    top_nav_bar_button('rig_conf.cgi','','','Configuration','s0','rig,'.$rig->name.',1');
	}
	else
	{		
	    if ($rig->is_iv)
	    {
		top_no_button;
	    }
	    else
	    {
		top_nav_bar_button('show_rig_values.cgi','','','Show current values','s2','rig,'.$rig->name.',1');
	    }
	}
	if ($readmode)
	{
	    top_nav_bar_button('iv_setup.cgi','','','Setup i-V curve parameters','s3','rig,'.$rig->name.',1');
	}
	else
	{
	    top_no_button;
	}
	top_nav_bar_button('main.cgi','','','To menu','s4');
	end_top_bar;
	param(-name=>'action',-value=>'')  unless (defined(param('action')));
	$_ = param('action');
      CASE:
	{
	    ### Insert case statements ###
	    if (!$readmode) {&main_page;last CASE;};
	    &main_page;
	}
	print_end;
    }
    else 
    {
	&not_auth;
    }
}

$rig = undef;    
exit;



sub main_page
{
    print '<center>';
    print '<table border=0 cellspacing=5 cellpadding=0 style="width:100%"><tr height=20>';
    if ($rig->list_devices('PID'))
    {
	print '<td width="5%"></td>';
	print '<td width="18%"></td>';
	print '<td width="18%"></td>';
	print '<td width="18%"></td>';
	print '<td width="18%"></td>';
	print '<td width="18%"></td>';
	print '<td width="5%"></td>';
	print '</tr>';
	print '<tr border=1><td colspan=7 class="heading">';
    }
    else
    {
	print '<td width="10%"></td>';
	print '<td width="20%"></td>';
	print '<td width="20%"></td>';
	print '<td width="20%"></td>';
	print '<td width="20%"></td>';
	print '<td width="10%"></td>';
	print '</tr>';
	print '<tr border=1><td colspan=6 class="heading">';
    }
    print "Main menu for rig ".$rig->name;
    print '</td></tr>';
    print start_form(-name=>'data');
    print hidden(-name=>'isactive',-value=>1,-force=>1);
    my @tnames = $rig->get_cv_list('userinterface','temp');
    my @gnames = $rig->get_cv_list('userinterface','gas');
    my @vnames = $rig->get_cv_list('userinterface','voltage');
    my @cnames = $rig->get_cv_list('userinterface','current');
    my $max = 0;
    $max = @tnames if (@tnames > $max);
    $max = @gnames if (@gnames > $max);
    $max = @vnames if (@vnames > $max);
    $max = @cnames if (@cnames > $max);
    for (my $x = 0; $x < $max; $x++)
    {
        print '<tr><td></td>';
        if ($x < @tnames)
        {
	    print "<td id='$tnames[$x]'>";
            my $title = $rig->get_cv('temp_'.$tnames[$x],'title');
            if ($title)
            {
                print "$title : ";
            }
            else
            {
                print "$tnames[$x] : ";
            }
	    print '</td>';
        }
	else
	{
	    print '<td></td>';
	}
        if ($x < @gnames)
        {
	    print "<td id='$gnames[$x]'>";
            my $title = $rig->get_cv('gas_'.$gnames[$x],'title');
            if ($title)
            {
                print "$title : ";
            }
            else
            {
                print "$gnames[$x] : ";
            }
	    print '</td>';
        }
	else
	{
	    print '<td></td>';
	}
        if ($x < @vnames)
        {
	    print "<td id='$vnames[$x]'>";
            my $title = $rig->get_cv('gas_'.$vnames[$x],'title');
            if ($title)
            {
                print "$title : ";
            }
            else
            {
                print "$vnames[$x] : ";
            }
	    print '</td>';
        }
	else
	{
	    print '<td></td>';
	}
        if ($x < @cnames)
        {
	    print "<td id='$cnames[$x]'>";
            my $title = $rig->get_cv('gas_'.$cnames[$x],'title');
            if ($title)
            {
                print "$title : ";
            }
            else
            {
                print "$cnames[$x] : ";
            }
	    print '</td>';
        }
	else
	{
	    print '<td></td>';
	}
    }
    print end_form();

    if ($usermode && RFC::RFCCGI::rig_auth())
    {
	print '<tr><td></td><td>';
	if ($rig->list_devices_control('tempcontrol'))
	{
	    print start_form(-class=>'form',-name=>'temperature');
	    print button(-name=>'dummym',-value=>'Set temperature',-class=>'main0',-OnClick=>"RFC_open('set_temp.cgi',450,250,300,200)");
	    print hidden(-name=>'device',-value=>'',-force=>'1');
	    print hidden(-name=>'temp',-value=>'25',-force=>'1');
	    print hidden(-name=>'ramp',-value=>'60',-force=>'1');
	    print_hidden();
	    print end_form;
	}
	if ($rig->list_devices_control('relay') || $rig->list_devices_control('water'))
	{
	    print start_form(-class=>'form',-name=>'relay');
	    print button(-name=>'dummym',-value=>'Set relay status',-class=>'main1',-OnClick=>"RFC_open('set_relay.cgi',450,200,300,200)");
	    print hidden(-name=>'device',-value=>'',-force=>'1');
	    print hidden(-name=>'status',-value=>'',-force=>'1');
	    print_hidden();
	    print end_form;	    
	}
	print '</td><td>';
	if ($rig->list_devices_control('gas'))
	{
	    print start_form(-class=>'form',-name=>'gas');
	    print button(-name=>'dummym',-value=>'Set gas flow',-class=>'main2',-OnClick=>"RFC_open('set_gas.cgi',450,200,300,200)");
	    print hidden(-name=>'device',-value=>'',-force=>'1');
	    print hidden(-name=>'flow',-value=>'',-force=>'1');
	    print_hidden();
	    print end_form;	    
	}
	if ($rig->list_devices_control('multiplex'))
	{
	    print start_form(-class=>'form',-name=>'multiplex');
	    print button(-name=>'dummym',-value=>'Set gas multiplexer',-class=>'main3',-OnClick=>"RFC_open('set_multiplex.cgi',450,200,300,200)");
	    print hidden(-name=>'device',-value=>'',-force=>'1');
	    print hidden(-name=>'gas',-value=>'',-force=>'1');
	    print_hidden();
	    print end_form;	    
	}
	print '</td><td>';
	my @list = $rig->list_devices_control('PSU');
	my $isvolt = 0;
	my $iscurr = 0;
	foreach my $n ($rig->list_devices_control('analog'))
	{
	    my $dev = $rig->get_device("analog",$n);
	    $dev->init();
	    $isvolt = 1 if ($dev->mode() && $dev->mode() =~ /V/i);
	    $iscurr = 1 if ($dev->mode() && $dev->mode() =~ /A/i);
	}
	if (@list || $isvolt || $iscurr)
	{
	    print start_form(-class=>'form',-name=>'PSU');
	    print button(-name=>'dummym',-value=>'Set DC current',-class=>'main4',-OnClick=>"RFC_open('set_current.cgi',450,200,300,200)") if (@list || $iscurr);
	    print button(-name=>'dummy2',-value=>'Set DC voltage',-class=>'main5',-OnClick=>"RFC_open('set_voltage.cgi',450,200,300,200)") if (@list || $isvolt);
	    print hidden(-name=>'device',-value=>'',-force=>'1');
	    print hidden(-name=>'current',-value=>'',-force=>'1');
	    print hidden(-name=>'voltage',-value=>'',-force=>'1');
	    print_hidden();
	    print end_form;
	}
	print '</td><td>';
	if ($rig->list_devices_control('PSU'))
	{
	    if ($rig->is_iv)
	    {
		print button(-name=>'dummym',-value=>'Stop running iV curve',-class=>'main6',-OnClick=>'iv_control(this)',-id=>'iv_control_btn');
	    }
	    else
	    {
		print button(-name=>'dummym',-value=>'Start iV curve',-class=>'main6',-OnClick=>'iv_control(this)',-id=>'iv_control_btn');
	    }
	}
	print start_form(-class=>'form',-name=>'is');
	print button(-name=>'dummym',-value=>'Run impedance',-class=>'main7',-OnClick=>"RFC_open('set_imp.cgi',450,250,300,200)");
	print_hidden();
	print hidden(-name=>'server',-value=>'',-force=>'1');
	print hidden(-name=>'port',-value=>'',-force=>'1');
	print hidden(-name=>'impuser',-value=>'',-force=>'1');
	print hidden(-name=>'session',-value=>'',-force=>'1');
	print hidden(-name=>'comp',-value=>'',-force=>'1');
	print end_form;
	print '</td><td>';
	if ($rig->list_devices('PID'))
	{
	    print start_form(-class=>'form',-name=>'pid');
	    print button(-name=>'dummym',-value=>'PID setpoint',-class=>'main0',-OnClick=>"RFC_open('set_pid.cgi',450,250,300,200)");
	    print hidden(-name=>'device',-value=>'',-force=>'1');
	    print hidden(-name=>'setpoint',-value=>'',-force=>'1');
	    print end_form;
	}
	print '</td></tr>'
    }
    print '<tr><td height=20></td></tr>';
    my $message = $rig->get_cv('main','warning_message');
    if ($message)
    {
	if ($rig->list_devices('PID'))
	{
	    print '<tr><th colspan=7 class="Alert">';
	}
	else
	{
	    print '<tr><th colspan=6 class="Alert">';
	}
	print $message;
	print '</td></tr>';
	print '<tr><td height=20></td></tr>';
    }
    print '</table>';

    print '<table border=0 cellspacing=5 cellpadding=0 style="width:100%"><tr height=20>';
    print '<td width="10%"></td>';
    print '<td width="20%"></td>';
    print '<td width="20%"></td>';
    print '<td width="20%"></td>';
    print '<td width="20%"></td>';
    print '<td width="10%"></td>';
    print '</tr>';


    print '<tr><td></td><td colspan=2 valign=top>';
    print 'Cell information file:';
    print '</td><td colspan=2 valign=top>';
    print 'Program log:';
    print '</td></tr>';
    print '<tr><td></td><td colspan=2 valign=top>';
    print start_form(-name=>'info');
    print textarea(-name=>'info',-value=>'',-force=>'1',-rows=>'12',-class=>'textfield',);
    print end_form();
    print 'Errorlog:';
    print start_form(-name=>'errorlog');
    print textarea(-name=>'errorlog',-value=>'',-force=>'1',-rows=>'5',-class=>'textfield',);
    print end_form();
    print '</td><td colspan=2 valign=top>';
    print start_form(-name=>'proglog');
    print textarea(-name=>'proglog',-value=>'',-force=>'1',-rows=>'12',-class=>'textfield');
    print end_form();
    RFC::RFCCGI::print_js_validate('sofc','comments');
    print 'Additional information:';
    print start_form(-name=>'new_comment');
    print textarea(-name=>'comments',-value=>'',-force=>'1',-rows=>'3',-class=>'textfield');
    print button(-name=>'new_comm',-value=>'Add comment',-class=>'s3',-onclick=>'add_comment()');
    print end_form();
    print '</td></tr>';
    print '</table>';
    my $user = get_user();

    print <<END;

<script language="javascript">

/* Variables and functions for controling ivcurve popups etc */

window.onblur = set_inactive;
window.onfocus = set_active;
var ispopup = false;
var is_logged_in = true;
var username = '$user';

function check_status()
{
    raw_ajax('ajax.cgi','?action=is_logged_in&name='+username,set_status);
}

function set_status(res)
{
    var reg = /YES/;
    if (!reg.test(res))
    {   
	is_logged_in = false;
	document.logout.submit();
    }
}



function set_active()
{
    document.data.isactive.value = '1';
}

/* Ugly hack for IE wrongly triggering window onblur when selecting element inside page..... */

function ie_inactive()
{
    if (bIsMSIE && (active_element != document.activeElement)) {
	active_element = document.activeElement;
    }
    else 
    {	    
	document.data.isactive.value = '0';
    }
}

function set_inactive()
{
    if (navigator.appName == "Microsoft Internet Explorer") {
	active_element = document.activeElement;
	bIsMSIE = true;
	document.onfocusout = ie_inactive;
	return false;
    }
    ie_inactive();
}

/* end IE hack */

function iv_popup()
{
    if (document.data.isactive.value == 0 || ispopup == true)
    {
	return false;
    }
    ispopup = true;
    RFC_open('ivfig.cgi',510,380,100,400);
}

/* End iv popup functions */

function load_isiv()
{
    run_ajax('rig_ajax_run.cgi','&action=is_iv',set_iv);
    setTimeout("load_isiv()",10000);    
}

function set_iv(res)
{
    var btn = document.getElementById('iv_control_btn');
    if (btn == null)
	return false;
    if (res == 1)
    {
	btn.value = "Stop running iV curve";
	iv_popup();
    }
    else
    {
	btn.value = "Start iV curve";
	ispopup = false;
    }
}

function iv_control(btn)
{
    var val = btn.value;
    var reg = /Stop/;
    if (reg.test(val))
    {
	var ok = confirm("Stop running iV curve?");
	if (ok)
	{
	    run_ajax('rig_ajax_run.cgi','&action=stop_iv',set_temp_ok);
	}
    }
    else
    {
	run_ajax('rig_ajax_run.cgi','&action=start_iv',set_temp_ok);
    }
}


function set_PSU_go()
{
    var dev = document.PSU.device.value;
    var t = document.PSU.current.value;
    run_ajax('rig_ajax_run.cgi','&action=set_current&dev='+dev+'&current='+t,set_temp_ok);
}

function set_PSU_volt_go()
{
    var dev = document.PSU.device.value;
    var t = document.PSU.voltage.value;
    run_ajax('rig_ajax_run.cgi','&action=set_voltage&dev='+dev+'&voltage='+t,set_temp_ok);
}

function set_temp_go()
{
    var dev = document.temperature.device.value;
    var t = document.temperature.temp.value;
    var r = document.temperature.ramp.value;
    run_ajax('rig_ajax_run.cgi','&action=set_temp&dev='+dev+'&temp='+t+'&ramp='+r,set_temp_ok);
}

function set_relay_go()
{
    var dev = document.relay.device.value;
    var t = document.relay.status.value;
    run_ajax('rig_ajax_run.cgi','&action=set_relay&dev='+dev+'&status='+t,set_temp_ok);
}

function set_multiplex_go()
{
    var dev = document.multiplex.device.value;
    var g = document.multiplex.gas.value;
    run_ajax('rig_ajax_run.cgi','&action=set_multiplex&dev='+dev+'&gas='+g,set_temp_ok);
}


function set_gas_go()
{
    var dev = document.gas.device.value;
    var f = document.gas.flow.value;
    run_ajax('rig_ajax_run.cgi','&action=set_gas&dev='+dev+'&flow='+f,set_temp_ok);
}

function set_pid_go()
{
    var dev = document.pid.device.value;
    var f = document.pid.setpoint.value;
    run_ajax('rig_ajax_run.cgi','&action=set_pid&dev='+dev+'&setpoint='+f,set_temp_ok);
}

function set_imp_go()
{
    var server = document.is.server.value;
    var p = document.is.port.value;
    var u = document.is.impuser.value;
    var s = document.is.session.value;
    var cp = document.is.comp.value;
    run_ajax('rig_ajax_run.cgi','&action=set_imp&server='+server+'&port='+p+'&impuser='+u+'&session='+s+'&comp='+cp,set_temp_ok);
}

function set_temp_ok(t)
{
    alert(t);
    /* Sumbit the login info to reset logout timer */
    document.login_info.submit();
}

function set_log(t)
{
    document.proglog.proglog.value = t;
    document.proglog.proglog.scrollTop = document.proglog.proglog.scrollHeight;
}

function set_info(t)
{
    document.info.info.value = t;
}

function set_error(t)
{
    document.errorlog.errorlog.value = t;
}

function set_data(t)
{
    var arr = t.split("\\n");
    for (var i = 0; i < arr.length;i++)
    {
	if (arr[i] != null && arr[i] != "")
	{
	    var item = arr[i].split("\\t");
	    var element = document.getElementById(item[0]);
	    var str = element.innerHTML;
	    var old = str.split(":");
	    var newd = old[0]+': '+item[1];
	    element.innerHTML = newd;
	}
    }
}

function load_log()
{
    if (is_logged_in)
    {
	check_status();
	run_ajax('rig_ajax_run.cgi','&action=show_log',set_log);
	setTimeout("load_log()",50000);
    }
}

function load_data()
{
    if (is_logged_in)
    {
	run_ajax('UIdata.cgi','&action=data',set_data);
	setTimeout("load_data()",120000);
    }
}

function load_info()
{
    if (is_logged_in)
    {
	run_ajax('rig_ajax_run.cgi','&action=show_info',set_info);
	setTimeout("load_info()",300000);
    }
}

function load_error()
{
    if (is_logged_in)
    {
	run_ajax('rig_ajax_run.cgi','&action=show_error',set_error);
	setTimeout("load_error()",300000);
    }
}



function add_comment()
{
    var text = document.new_comment.comments.value;
    if (checkinput)
    {
	document.new_comment.comments.value = '';
	/* Translate all newlines to literal '\n' strings to make them
	    survive the XHTML transfer..... */
	text = text.replace(\/\\n\/mg,"\\\\n");
	run_ajax('rig_ajax_run.cgi','&action=add_info&comments='+text,set_info);
    }
}



</script>

END


}    

